//
// $Id: ObjectReplaceTable.m,v 1.6 2007/03/06 20:42:19 will_mason Exp $
//
// vi: set ft=objc:

/*
 * ObjectiveLib - a library of containers and algorithms for Objective-C
 *
 * Copyright (c) 2004-2007
 * Will Mason
 *
 * Portions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Copyright (c) 1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Copyright (c) 1997
 * Moscow Center for SPARC Technology
 *
 * Copyright (c) 1999 
 * Boris Fomitchev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author at will_mason@users.sourceforge.net.
 */

#import "ObjectReplaceTable.h"
#import "Macros.h"
#import "RunTime.h"
#import <string.h>
#import <stdlib.h>

@implementation OLObjectReplaceTable

- (id) init
{
    [super init];
    refs = [[OLPointerRefTable alloc] init];
    repCapacity = [refs capacity];
    reps = objc_malloc(repCapacity * sizeof(id));
    return self;
}

#if defined(OL_NO_OPENSTEP)
- (id) free
#else
- (void) dealloc
#endif
{
    OBJ_RELEASE(refs);
    objc_free(reps);
    SUPER_FREE;
}

- (void) assign: (id)object replacement: (id)rep
{
    uint32_t index = [refs lookUp: object];

    while (index >= repCapacity)
        [self grow];
    reps[index] = rep;
}

- (void) grow
{
    unsigned newCapacity = (repCapacity << 1) + 1;
    id* newReps = objc_malloc(newCapacity * sizeof(id));

    memcpy(newReps, reps, repCapacity * sizeof(id));
    repCapacity = newCapacity;
    objc_free(reps);
    reps = newReps;
}

- (id) lookUp: (id)object
{
    uint32_t index = [refs lookUp: object];
    return (index == UINT32_MAX) ? object : reps[index];
}

@end
